# coding: utf-8
require "./header-footer.rb"

class Rational
  def to_tex(op=false, amplify=1)
    if denominator==1
      return (op && self>=0 ? "+" : "")+numerator.to_s
    else
      s = numerator<0 ? "-" : (op ? "+" : "")
      return "#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}"
    end
  end  
end

def randRat
    r=nil
    10.times {
        r=Rational(rand(4)+1, rand(9)+1)*(rand(2)*2-1)
        return r if r.denominator>1
    }
    return r
end

def poly_to_tex(exp, coeff)
    res = ""
    exp.each.with_index{|e,i|
        if coeff[i]!=0
            if coeff[i]==-1
                res+="-"
            elsif coeff[i]!=1
                res+=coeff[i].to_tex(i!=0)
            else
                res += i!=0 ? "+" : ""
            end
            res+="x^{#{e}}" if e>1
            res+="x" if e==1
        end
    }
    return res                   
end

def polyderive(exp, coeff)
    return [exp.map{|x| x==0 ?  0 : x-1}, coeff.map.with_index{|c,i| c*exp[i]}]
end

def polynome(terms=3)
    exp = (2..12).to_a.shuffle()[0...terms].sort.reverse
    coeff = Array.new(terms) {randRat()}
    ex = poly_to_tex(exp, coeff)
    coeff.map!.with_index{|c,i| c*exp[i]}
    exp.map!{|x| x-1}
    sol = poly_to_tex(exp,coeff)
    return [ex,sol]
end

def natlog()
    coeff = randRat()
    ex = coeff.to_tex+"\\ln(x)"
    sol = coeff.to_tex+"\\cdot \\frac{1}{x}"
    return [ex,sol]
end

def wurzel()
    coeff=randRat()
    ex = coeff.to_tex+"\\cdot \\sqrt{x}"
    coeff/=2
    sol = coeff.to_tex+"\\cdot \\frac{1}{\\sqrt{x}}"
    return [ex,sol]
end

def expallg()
    coeff=randRat()
    base = rand(4)+2
    ex = coeff.to_tex+"\\cdot #{base}^{x}"
    sol = coeff.to_tex+"\\cdot \\ln(#{base}) \\cdot #{base}^x"
    return [ex,sol]
end

def lnpoly(terms=3)
    exp = (0..4).to_a.shuffle()[0...terms].sort.reverse
    coeff = Array.new(terms) {randRat()}
    ex = "\\ln\\left("+poly_to_tex(exp, coeff)+"\\right)"
    d = polyderive(exp,coeff)
    sol = "\\frac{1}{"+poly_to_tex(exp, coeff)+"}\\cdot \\left("+poly_to_tex(d[0],d[1])+"\\right)"
    return [ex,sol]
end

def exppoly(terms=3)
    exp = (0..4).to_a.shuffle()[0...terms].sort.reverse
    coeff = Array.new(terms) {randRat()}
    ex = "\\mathrm{e}^{\\left("+poly_to_tex(exp, coeff)+"\\right)}"
    d = polyderive(exp,coeff)
    sol = "\\mathrm{e}^{\\left("+poly_to_tex(exp, coeff)+"\\right)}\\cdot \\left("+poly_to_tex(d[0],d[1])+"\\right)"
    return [ex,sol]
end

def sincos(terms=3, pick=rand(2))
  fn = ["\\cos", "\\sin"]
  df = ["-\\sin", "\\cos"]
  factor = randRat()
  exp = (0..4).to_a.shuffle()[0...terms].sort.reverse
  coeff = Array.new(terms) {randRat()}
  usep = pick==0
   ex = "#{factor.to_tex}#{usep ? "\\left(" : ""}#{fn[pick]}\\left("+poly_to_tex(exp, coeff)+"\\right)#{usep ? "\\right)" : ""}"
   d = polyderive(exp,coeff)
   sol = "#{factor.to_tex} \\cdot \\left(#{poly_to_tex(d[0],d[1])}\\right) \\cdot #{df[pick]}\\left("+poly_to_tex(exp, coeff)+"\\right)"
   if (pick==0)
     df[0] = "\\sin"
   end
   d[1].map!{|c| c*factor*(pick==0 ? -1 : 1)}
   usep = d[1].count{|c| c!=0}>1
   puts "#{d[1].inspect} => size = #{d[1].size}"
   sol += " = #{usep ? "\\left(" : ""}#{poly_to_tex(d[0],d[1])}#{usep ? "\\right)" : ""}\\cdot #{df[pick]}\\left("+poly_to_tex(exp, coeff)+"\\right)"
   return [ex,sol]
end


def ableiten(num=10)
  return Array.new(num) {
    es = [polynome, natlog, wurzel,expallg]
    ex = ""
    sol = ""
    %w(a b c d).each.with_index{|a,i|
      ex += "#{a}) $f(x)=#{es[i][0]}\\quad$ "
      sol += "#{a}) $f'(x)=#{es[i][1]}\\quad$ "
    }
    [ex,sol]
  }
end

def ableitenkettenregel(num=10)
  return Array.new(num) {
    es = [lnpoly, exppoly]
    ex = ""
    sol = ""
    %w(a b).each.with_index{|a,i|
      ex += "#{a}) $f(x)=#{es[i][0]}\\quad$ "
      sol += "#{a}) $f'(x)=#{es[i][1]}\\quad$ "
    }
    [ex,sol]
  }
end

def ableitentrigpoly(num=10)
  return Array.new(num) {
    es = [sincos(2,0), sincos(2,1)]
    ex = ""
    sol = ""
    %w(a b).each.with_index{|a,i|
      ex += "#{a}) $f(x)=#{es[i][0]}\\quad$ "
      sol += "#{a}) $f'(x)=#{es[i][1]}$ <br> "
    }
    [ex,sol]
  }
end


type = ARGV[0]
res = "" 
if (type.to_i==3)
  name = "trigpoly"
  res = "Leiten Sie von Hand und ohne Unterlagen ab:\n"
  res += header(name)
  res += ableitentrigpoly().inspect
  res += footer(name, " <br> ")+"\n"
  
elsif (type.to_i==2)
  name = "kettenregelvonpoly"
  res = "Leiten Sie von Hand und ohne Unterlagen ab:\n"
  res += header(name)
  res += ableitenkettenregel().inspect
  res += footer(name, " <br> ")+"\n"
  
else
  name = "ableiten"
  
  
  res = "Leiten Sie von Hand und ohne Unterlagen ab:\n"
  res += header(name)
  res += ableiten().inspect
  res += footer(name, " <br> ")+"\n"
end

xclip(res)
  
      
