#!/usr/bin/ruby
# coding: utf-8

# Use with:
# ruby bruch-add-mul-zahlen.rb N E [T]
#
# N: number of exercises
# E: Exercise
#      1: Bruch addieren, multiplizieren
#      2: Gleichungen erweitern (mit Addition, Multiplikation im Bruch)
#      3: Rationale Exponenten mit $x$ in der Basis
# T: Type (0 or 1)  (Nur für Ex 1)

require './header-footer.rb'

class Rational
  def to_tex(amplify=1, paren=false)
    p=(paren && (self<0)) ? ["\\left(", "\\right)"] : ["",""]
    if denominator==1
      return p[0]+numerator.to_s+p[1]
    else
      s = numerator<0 ? "-" : ""
      return "#{p[0]}#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}#{p[1]}"
    end
  end  
end

def tex2rat(s)
  s=s.gsub("\\cdot","*").gsub(":","/").gsub("\\left", "").gsub("\\right","").gsub("$","")
  s=s.gsub(/\\frac\{(.*?)\}\{(.*?)\}/,"Rational(\\1,\\2)").gsub("=",",")
  s
end

def gcd(a,b)
  return a if b==0
  return gcd(b, a%b)
end

def lcm(a,b)
  return a*b/gcd(a,b)
end

def gcds(a)
  if (a.size==2)
    return gcd(a[0],a[1])
  else
    return gcd(a[-1], gcds(a[0..-2]))
  end
end

def lcms(a)
  if  (a.size==2)
    return lcm(a[0],a[1])
  else
    return lcm(a[-1], lcms(a[0..-2]))
  end
end

def sum2tex(a,b,pm)
  cd = a.denominator*b.denominator/gcd(a.denominator,b.denominator)
  return a.to_tex(cd/a.denominator)+%w(- +)[pm]+b.to_tex(cd/b.denominator)
end

def randRat(nmin, nmax, dmin, dmax)
  r = nil
  50.times{
    r = Rational(rand(nmax+1)+nmin, rand(dmax+1)+dmin)
    if r.denominator>=dmin
      r*=(rand(2)*2-1)
      return r
    end
  }
  return r
end

# (frac +/- frac) */: frac
def ex1(type=rand(2))
  a,b,c,r = nil
  begin
    begin
      a = Rational(rand(8)+2, rand(5)+2)
    end until a.denominator != 1
    begin
      b = Rational(rand(8)+2, rand(5)+2)
    end until b.denominator != 1 && b.denominator!=a.denominator
    
    pm = rand(2)
    s = a+(pm*2-1)*b
    
    begin
      r = Rational(rand(8)+2, rand(5)+2)*(rand(2)*2-1)
    end until r.denominator != 1
    
    dm = rand(2)
    
    q = r/s
    
    if (dm==0)
      q = 1/q
    end
  end until s.numerator!=0 && q.denominator!=1 && q.numerator<100 && q.denominator<100
  if type==0
    # (a+-b)*q=r
    return "$$\\left("+a.to_tex+%w(- +)[pm]+b.to_tex(1,true)+"\\right)"+%w(: \\cdot)[dm]+q.to_tex(1,true)+"$$",
           "$$\\left("+sum2tex(a,b,pm)+"\\right)"+%w(: \\cdot)[dm]+q.to_tex+"="+
           s.to_tex+%w(: \\cdot)[dm]+q.to_tex+"="+
           r.to_tex+"$$"
  elsif type==1
    # r/q-+b = a
    return "$$"+r.to_tex+%w(\\cdot :)[dm]+q.to_tex(1,true)+%w(+ -)[pm]+b.to_tex(1,true)+"$$",
           "$$"+(dm==0 ? r*q : r/q).to_tex+%w(+ -)[pm]+b.to_tex+"="+
           sum2tex(dm==0 ? r*q : r/q, b, 1-pm)+"="+
           a.to_tex+"$$"
  end
  
end

def ex_ehochx
  ex = Hash.new
  common = 0
  50.times{|i|
    ex['res'] = randRat(4,12,3,12)
    ex['mul'] = randRat(2,9,2,9)
    ex['paren'] = ex['res']/ex['mul']
    ex['sub'] = randRat(2,9,3,10)
    ex['add'] = randRat(2,10,2,10)
    ex['fst'] = ex['paren']-ex['add']+ex['sub']
    common = lcms([ex['fst'].denominator, ex['add'].denominator, ex['sub'].denominator])
    break if (ex['paren'].denominator <=100) && (ex['fst'].denominator <=100) && common<=100 && (ex['fst'].denominator != ex['add'].denominator || ex['add'].denominator != ex['sub'].denominator) && ex['paren'].numerator.abs*common/ex['paren'].denominator<=150
    STDERR.puts "**********************\n** Shit happened! **\n*********************" if i==49
  }
  return "$\\displaystyle \\left(\\frac{\\mathrm{e}^{#{ex['fst'].to_tex}\\cdot x} \\cdot \\left(\\mathrm{e}^x\\right)^{#{ex['add'].to_tex}}}{\\sqrt[#{ex['sub'].denominator}]{\\mathrm{e}^{#{ex['sub'].numerator}\\cdot x}}}\\right)^{#{ex['mul'].to_tex}}$",
         "$\\displaystyle \\left(\\frac{\\mathrm{e}^{#{ex['fst'].to_tex}\\cdot x} \\cdot \\left(\\mathrm{e}^x\\right)^{#{ex['add'].to_tex}}}{\\sqrt[#{ex['sub'].denominator}]{\\mathrm{e}^{#{ex['sub'].numerator}\\cdot x}}}\\right)^{#{ex['mul'].to_tex}} = " +
         "\\left(\\frac{\\mathrm{e}^{#{ex['fst'].to_tex}\\cdot x} \\cdot \\mathrm{e}^{#{ex['add'].to_tex}\\cdot x}}{\\mathrm{e}^{#{ex['sub'].to_tex}\\cdot x}}\\right)^{#{ex['mul'].to_tex}} = " +
         "\\left(\\mathrm{e}^{x \\cdot \\left(#{ex['fst'].to_tex} + #{ex['add'].to_tex(1,true)} - #{ex['sub'].to_tex(1,true)}\\right)} \\right)^{#{ex['mul'].to_tex}} = $\n$" +
         "\\left(\\mathrm{e}^{x \\cdot \\left(#{ex['fst'].to_tex(common/ex['fst'].denominator)} + #{ex['add'].to_tex(common/ex['add'].denominator,true)} - #{ex['sub'].to_tex(common/ex['sub'].denominator,true)}\\right)} \\right)^{#{ex['mul'].to_tex}} = " +
         "\\left(\\mathrm{e}^{x \\cdot #{ex['paren'].to_tex(common/ex['paren'].denominator,true)}} \\right)^{#{ex['mul'].to_tex}} = " +
         "\\left(\\mathrm{e}^{x \\cdot #{ex['paren'].to_tex(1,true)}} \\right)^{#{ex['mul'].to_tex}} = $\n$" +
         "\\mathrm{e}^{x \\cdot #{ex['paren'].to_tex(1,true)} \\cdot #{ex['mul'].to_tex(1,true)}} = " +
         "\\mathrm{e}^{#{ex['res'].to_tex} \\cdot x}$"
  
 "" 
end

# Das gleiche wie ex1, aber mit Potenzen und den entsprechenden Brüchen in den Exponenten.
def ex3(type=rand(2))
  a,b,c,r = nil
  begin
    begin
      a = Rational(rand(8)+2, rand(5)+2)
    end until a.denominator != 1
    begin
      b = Rational(rand(8)+2, rand(5)+2)
    end until b.denominator != 1 && b.denominator!=a.denominator
    
    pm = rand(2)
    s = a+(pm*2-1)*b
    
    begin
      r = Rational(rand(8)+2, rand(5)+2)*(rand(2)*2-1)
    end until r.denominator != 1
    
    dm = rand(2)
    
    q = r/s
    
    if (dm==0)
      q = 1/q
    end
  end until s.numerator!=0 && q.denominator!=1 && q.numerator<100 && q.denominator<100
  if type==0
    # (a+-b)*q=r
    return "$\\left(x^{"+a.to_tex+"}"+%w(: \\cdot)[pm]+" x^{"+b.to_tex+"}\\right)^{"+(q**([-1,1][dm])).to_tex+"}$",
#           "$$\\left("+sum2tex(a,b,pm)+"\\right)"+%w(: \\cdot)[dm]+q.to_tex+"="+
#           s.to_tex+%w(: \\cdot)[dm]+q.to_tex+"="+
           "$x^{"+r.to_tex+"}$"
  elsif type==1
    # r/q-+b = a
    return "$\\left(x^{"+r.to_tex+"}\\right)^{"+(q**[1,-1][dm]).to_tex+"}"+%w(\\cdot :)[pm]+" x^{"+b.to_tex+"}$",
#           "$$"+(dm==0 ? r*q : r/q).to_tex+%w(+ -)[pm]+b.to_tex+"="+
#           sum2tex(dm==0 ? r*q : r/q, b, 1-pm)+"="+
           "$x^{"+a.to_tex+"}$"
  end
  
end

def ex2
  while true
    vars = ('a'..'z').to_a.shuffle[0..2]
    nenner = [3,4,5,6,8,9,10,12,15,16,18,20].shuffle[0..2]
    coeff = Array.new(3){Array.new(2){ rand(5)+2 }}
    2.times{|i|
      puts  "#{coeff[i]} und #{nenner[i]}"
      k = gcds(coeff[i]+[nenner[i]])
      puts "  -> #{k}"
      nenner[i]/=k
      coeff[i].map{|c| c/k}
    }
    k = gcd(coeff[2][0]*coeff[2][1],nenner[2])
    nenner[2]/=k
    k2 = gcd(coeff[2][0],k)
    coeff[2][0]/=k2
    coeff[2][1]/=(k/k2)
    next if nenner.include?(1) || coeff.select{|c| c[0]==1}.size>0 || coeff[2][1]==1

    pm = ['+','-'][rand(2)]
    
    t = Array.new(3) {|i|
      "\\frac{#{coeff[i][0]}#{vars[i]}#{i==2 ? " \\cdot " : " + "}#{coeff[i][1]}}{#{nenner[i]}}"
    }
    perm = (0..2).to_a.shuffle
    
    aufgabe = "$#{t[perm[0]]}#{pm}#{t[perm[1]]} = #{t[perm[2]]}$"
    
    e = lcms(nenner)

    next if e>50
    
    lsg = "Erweitern mit #{e}: "
    
    2.times{|i|
      coeff[i].map!{|c| c*=e/nenner[i] }
    }
    l = Array.new(3) {|i|
      if i<2
        "#{coeff[i][0]}#{vars[i]}+#{coeff[i][1]}"
      else
        "#{coeff[i][0]*coeff[i][1]*e/nenner[2]}#{vars[i]}"
      end
    }
    left = (pm=="-" && perm[1]!=2) ? "(" : ""
    right = (left == "(") ?  ")" : ""
    lsg += "$#{l[perm[0]]}#{pm}#{left}#{l[perm[1]]}#{right} = #{l[perm[2]]}$"
    return aufgabe, lsg
  end
end

def vec2tex(v)
  return "\\begin{pmatrix} #{v.map{|vv| vv.to_tex()}.join("\\\\ ")} \\end{pmatrix}"
end

def okvec(v)
  return v.all?{|vv| vv.denominator < 140 && vv.numerator.abs<140}
end

def ex_vektoren
  while true
    vres = Array.new(3){ randRat(3,10,3,10) }
    vdiff = Array.new(3){|i|
      r = randRat(3,10,5,13)
      50.times{
        r = randRat(3,10,5,13)
        break if (r.denominator!=vres[i].denominator)
      }
      r
    }
    vstart = Array.new(3) {|i| vres[i]+vdiff[i]}
    l = Array.new(2) { randRat(3,8,3,9) }
    vstartl = vstart.map{|v| v/l[0]}
    vdiffl = vdiff.map{|v| v/l[1]}
    
    ex = "$#{l[0].to_tex()} \\cdot #{vec2tex(vstartl)} - #{l[1].to_tex(1,true)} \\cdot #{vec2tex(vdiffl)}$"
    sol = "$#{vec2tex(vstart)} - #{vec2tex(vdiff)} = #{vec2tex(vres)}$"
    if okvec(vstartl) && okvec(vdiffl) && okvec(vstart) && okvec(vdiff) && okvec(vres)
      return ex,sol
    end
  end
end

n = [ARGV[0].to_i,1].max

ex = ARGV[1].to_i

type = ARGV[2] ? ARGV[2].to_i : -1

if (ex==1)
  
  e = Array.new(n){ex1(type==-1 ? rand(2) : type)}
  
  text = "Berechnen Sie:\n"
  text += e.map{|ex| "  - #{ex[0]}\n"}.join("")
  text += "\n<hidden Lösungen>\n"
  text += e.map{|ex| "  - #{ex[1]}\n"}.join("")
  text += "</hidden>"

  name = "bruchrechnen2"
  res = "Berechnen Sie:\n"
  res += header(name)
  res += e.inspect
  res += footer(name, " &nbsp; &nbsp; ")+"\n"
  puts text
  
  puts res
  
  IO.popen("xclip", "w") {|f|
    f.puts res
  }
  puts "\n\n    -> Copied to X-Buffer (using xclip)"  
  
  
  # Verification
  # puts e.map{|ex| "["+ex.map{|t| tex2rat(t)}.join(",")+"]"}
elsif ex==2

  e = Array.new(n){ex2()}
  
  text = "Erweitern Sie folgende Gleichungen mit der kleinstmöglichen Zahl so, dass alle Nenner wegfallen. Sie brauchen nicht zusammenzufassen:\n"
  text += e.map{|ex| "  - #{ex[0]}\n"}.join("")
  text += "\n<hidden Lösungen>\n"
  text += e.map{|ex| "  - #{ex[1]}\n"}.join("")
  text += "</hidden>"
 
  puts text
elsif (ex==3)
  
  e = Array.new(n){ex3(type==-1 ? rand(2) : type)}
  
  text = "Vereinfachen Sie:\n"
  text += e.map{|ex| "  - #{ex[0]}\n"}.join("")
  text += "\n<hidden Lösungen>\n"
  text += e.map{|ex| "  - #{ex[1]}\n"}.join("")
  text += "</hidden>"
  
  # puts text
  name = "bruchexp"
  res = "Vereinfachen Sie und schreiben Sie als eine Potenz von $x$\n"
  res += header(name)
  res += e.inspect
  res += footer(name, " &nbsp; &nbsp; ")+"\n"

  puts res
  
  IO.popen("xclip", "w") {|f|
    f.puts res
  }
  puts "\n\n    -> Copied to X-Buffer (using xclip)"

  # Verification
  # puts e.map{|ex| "["+ex.map{|t| tex2rat(t)}.join(",")+"]"}
elsif (ex==4)
  
  e = Array.new(n){ex_ehochx()}
  
  # puts text
  name = "ehochxbrueche"
  res = "Vereinfachen Sie:\n"
  res += header(name)
  res += e.inspect
  res += footer(name, " &nbsp; &nbsp; ")+"\n"

  xclip(res)
  
elsif (ex==5)
  
  e = Array.new(n){ex_vektoren()}
  
  # puts text
  name = "vecmulsum"
  res = "Berechnen Sie (alle Brüche im Resultat vollständig gekürzt angeben):\n"
  res += header(name)
  res += e.inspect
  res += footer(name, " <br><hr> ")+"\n"

  xclip(res)
end
