#!/usr/bin/ruby
# coding: utf-8
#

require '../header-footer.rb'

class Rational
    def to_tex(op=false, amplify=1)
        if denominator==1
            return (op && self>=0 ? "+" : "")+numerator.to_s
        else
            s = self<0 ? "-" : (op ? "+" : "")
            return "#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}"
        end
    end  
    
    def nice(op=false) 
        if denominator==1
            return (op && self>=0 ? "+" : "")+numerator.to_s
        else
            s = self<0 ? "-" : (op ? "+" : "")
            return "#{s}#{numerator.abs}/#{denominator}"
        end        
    end
end


def randlin
	m=[Rational(3,2),2,0.5].map{|x| Rational(x)}
	q=[Rational(1,2), 1].map{|x| Rational(x)}
	m = (rand(2)*2-1)*m[rand(m.size)]
	q = (rand(2)*2-1)*q[rand(q.size)]
	ex = "#{m.to_tex} \\cdot x #{q.to_tex(true)}"
    sol = ["#{m.to_f}*x+#{q.to_f}", "#{m.nice}*x #{q.nice(true)}"]
    return [ex,sol, "Steigung $#{m.to_tex}$ durch den $y$-Achsenabschnitt $#{q.to_tex}$"]
end

def randquad
    a = [Rational(1),2,0.5, 0.25].map{|x| Rational(x)}
    a = (rand(2)*2-1)*a[rand(a.size)]
    x = rand(5)-2;
    y = (rand(2)+1)*(a>0 ? -1 : 1)
    # a*(t-x)^2+y
    b = -2*x*a
    c = a*x*x+y
    x = Rational(x)
    y = Rational(y)
    ex = "#{a.to_tex}\\cdot\\left(x#{(-x).to_tex(true)}\\right)^2 #{y.to_tex(true)}"
    sol = ["#{a.to_f}*x*x+#{b.to_f}*x+#{c.to_f}", "#{a.nice}*(x#{(-x).nice(true)})^2#{y.nice(true)}"]
    return [ex, sol, "Parabel mit Öffnungsfaktor $#{a.to_tex}$ und Scheitel $S=(#{x.to_tex},\\, #{y.to_tex})$"]
end

def randexp
    a = [Rational(2,1), Rational(1,2)][rand(2)]
    x = rand(2)*2-1
    x = Rational(x)
    pm = rand(2)==0 ? "-" : ""
    ex = "#{pm}\\left(#{a.to_tex}\\right)^{x#{(-x).to_tex(true)}}"
    sol = ["#{pm}#{a.to_f}**(x-#{x.to_f})", "#{pm}(#{a.nice})^{(x#{(-x).nice(true)})}"]
    return [ex, sol, "Exponentialfunktion mit Basis $#{a.to_tex}$, #{pm=='-' ? 'an $y$-Achse gespiegelt, ' : ""} um #{x.to_tex} in $x$-Richtung verschoben."]
end

def linquadexp(num=10)
  name = "linquadexp"
  fulltext = "Zeichnen Sie die Graphen der folgenden Funktionen in ein Koordinatensystem mit Einheit 4 Häuschen für $x$ im Bereich $-3$ bis $3$ und $y$ ebenfalls von $-3$ bis $3$.\n"
  fulltext += header(name)
  ex = Array.new(num) {[randlin(), randquad(), randexp()]}
  res = ""
  url = "https://fginfo.ksbg.ch/~ivo/miniaufgaben/"
  scp = "ivo@fginfo:public_html/miniaufgaben/."
  blob = []
  ex.each.with_index{|e,i|
    gnuplot = "set terminal svg size 400,400 dynamic\n# Axes label\nset xlabel 'x'\nset ylabel 'y'\n# Axes ranges\nset xrange [-3:3]\nset yrange [-3:3]\n# Axes tics\nset xtics 1\nset ytics 1\nset grid xtics ytics\nshow grid\n"
    names = %w(f g h)
    res = ""
    gnuplot += "plot "
    e.each.with_index{|f,j|
      gnuplot += "#{f[1][0]} title '#{f[1][1]}'"
      gnuplot += j==2 ? "\n" : ", "
      res+="$#{names[j]}(x) = #{f[0]}$, &nbsp"
    }
    File.write("fct.gnuplot",gnuplot)
    file = "#{name}#{i}.svg"
    `gnuplot fct.gnuplot > #{file}`
    `scp #{file} #{scp}`
    sol = e.map{|f| f[2]}.join("<br>")
    blob.push([res, sol+"<br><img src='#{url}#{file}?x=#{rand(1000000)}'>"])
  }
  fulltext += blob.inspect
  fulltext += footer(name, " <br> ", " <hr> ")+"\n"
  xclip(fulltext)
end

def parabelmanipex()
  a = [0.5, 0.25].map{|x| Rational(x)}
  a = (rand(2)*2-1)*a[rand(a.size)]
  x = Rational(rand(4)-2,1);
  x+= Rational(1,1) if x==0
  y = Rational((rand(4)+2),2)*(a>0 ? -1 : 1)
  ex = "$f(x)=x^2,\\qquad$ $g(x)=#{a.to_tex}\\cdot x^2,\\qquad$ $h(x)=#{a.to_tex}\\cdot \\left(x#{(-x).to_tex(true)}\\right)^2,\\qquad$ $k(x)=#{a.to_tex}\\cdot \\left(x#{(-x).to_tex(true)}\\right)^2#{y.to_tex(true)}$"
  sol = "#{a.to_f}*(x-(#{x.to_f}))*(x-(#{x.to_f}))+(#{y.to_f}) title 'k(x)=#{a.nice}(x#{(-x).nice(true)})^2#{y.nice(true)}' lt rgb 'red', #{a.to_f}*(x-(#{x.to_f}))*(x-(#{x.to_f})) title 'h(x)=#{a.nice}(x#{(-x).nice(true)})^2' lt rgb 'violet', #{a.to_f}*x*x title 'g(x)=#{a.nice}x^2' lt rgb 'brown', x*x title 'f(x)=x^2' lt rgb 'black'"
  return [ex, sol, "Normalparabel, Streckung in $y$-Richtung mit Faktor $#{a.to_tex}$, Verschiebung um $#{(x).to_tex}$ in $x$-Richtung, Verschiebung um $#{y.to_tex}$ in $y$-Richtung."]
end


def parabelmanip(num=10)
  name = "parabelmanip"
  fulltext = "Zeichnen Sie die Graphen der folgenden Funktionen in ein Koordinatensystem mit Einheit 4 Häuschen für $x$ im Bereich $-3$ bis $3$ und $y$ ebenfalls von $-3$ bis $3$.\n"
  fulltext += header(name)
  ex = Array.new(num) {parabelmanipex()}
  res = ""
  url = "https://fginfo.ksbg.ch/~ivo/miniaufgaben/"
  scp = "ivo@fginfo:public_html/miniaufgaben/."
  blob = []
  ex.each.with_index{|e,i|
    gnuplot = "set terminal svg size 400,400 dynamic\n# Axes label\nset xlabel 'x'\nset ylabel 'y'\n# Axes ranges\nset xrange [-3:3]\nset yrange [-3:3]\n# Axes tics\nset xtics 1\nset ytics 1\nset grid xtics ytics\nshow grid\n"
    gnuplot += "plot "
    gnuplot += "#{e[1]}"
    File.write("fct.gnuplot",gnuplot)
    file = "#{name}#{i}.svg"
    `gnuplot fct.gnuplot > #{file}`
    `scp #{file} #{scp}`
    blob.push([e[0], e[2]+"<br><img src='#{url}#{file}?x=#{rand(1000000)}'>"])
  }
  fulltext += blob.inspect
  fulltext += footer(name, " <br> ", " <hr> ")+"\n"
  xclip(fulltext)
end




if ARGV[0].to_i == 1
  linquadexp(10)
elsif ARGV[0].to_i == 2
  parabelmanip
else
  puts "Please specify type (1=linquadexp, 2=parabelmanip)"
end

