#!/usr/bin/ruby
# coding: utf-8

require './header-footer.rb'
require "matrix"

class Rational
  def to_tex(amplify=1, paren=false)
    p=(paren && (self<0)) ? ["\\left(", "\\right)"] : ["",""]
    if denominator==1
      return p[0]+numerator.to_s+p[1]
    else
      s = numerator<0 ? "-" : ""
      return "#{p[0]}#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}#{p[1]}"
    end
  end  
end

class Vector
  def to_tex(vec=true)
    return "\\begin{pmatrix} "+self.map{|x| x.to_tex()}.to_a.join(" \\\\ ")+"\\end{pmatrix} " if vec
    return "\\left("+self.map{|x| x.to_tex()}.to_a.join(",\\,")+"\\right)"
  end
end


def tex2rat(s)
  s=s.gsub("\\cdot","*").gsub(":","/").gsub("\\left", "").gsub("\\right","").gsub("$","")
  s=s.gsub(/\\frac\{(.*?)\}\{(.*?)\}/,"Rational(\\1,\\2)").gsub("=",",")
  s
end

def gcd(a,b)
  return a if b==0
  return gcd(b, a%b)
end

def lcm(a,b)
  return a*b/gcd(a,b)
end

def gcds(a)
  if (a.size==2)
    return gcd(a[0],a[1])
  else
    return gcd(a[-1], gcds(a[0..-2]))
  end
end

def lcms(a)
  if  (a.size==2)
    return lcm(a[0],a[1])
  else
    return lcm(a[-1], lcms(a[0..-2]))
  end
end

def sum2tex(a,b,pm)
  cd = a.denominator*b.denominator/gcd(a.denominator,b.denominator)
  return a.to_tex(cd/a.denominator)+%w(- +)[pm]+b.to_tex(cd/b.denominator)
end

def randRat(nmin, nmax, dmin, dmax)
  r = nil
  50.times{
    r = Rational(rand(nmax+1)+nmin, rand(dmax+1)+dmin)
    if r.denominator>=dmin && r.numerator>=nmin
      r*=(rand(2)*2-1)
      return r
    end
  }
  return r
end

def hasSquare?(i)
   r=2;
   while (r*r<=i)
       if i % (r*r) == 0
           return true
       end
       r+=1
   end
   return false
end

def wurzel(i)
   c = 1
   r=2
   while (r*r<=i)
       if (i% (r*r) == 0)
          c*=r
          i/=(r*r)
       else
           r+=1
       end
   end
   return c,i       
end

def tripel(max)
    res = []
    for x in 1..max
        for y in x..max
            for z in y..max
                l2 = x*x+y*y+z*z
                if l2 == (Math.sqrt(l2).to_i)**2
                    res.push(Vector.elements([Rational(x,1),Rational(y,1),Rational(z,1)]))
                end
            end
        end
    end
    return res
end


def gerade_kugel(trip)
  while true
    t = trip[rand(trip.size)]
    f = gcds(t.to_a)
    t/=f
    t = Array.new(2){Vector.elements(t.map{|x| x*(rand(2)*2-1)}.to_a.shuffle)}
    v = t[1]-t[0]
    next if v.any?{|x| x==0}
    puts "t: #{t.inspect}"
    f = gcds(v.to_a).abs
    v/=f
    puts "v: #{v.inspect}, f=#{f}"
    la = (rand(3)+1)
    a = t[0]-la*v
    lb = rand(2*f)+1
    lb+=1 if lb>=f
    b = t[0]+lb*v
    r = Rational(t[0].norm,1)
    lambda = [Rational(la,la+lb), Rational(la+f,la+lb)]
    puts "A,B #{[a,b].inspect},   la,lb = #{la}, #{lb},  lambda=#{lambda}"
    ex = "$A=#{a.to_tex(false)}$, $B=#{b.to_tex(false)}$, $r=#{r.to_tex}$."
    sol = "PD: $\\vec{OG}(\\lambda) = #{a.to_tex()} + \\lambda #{(b-a).to_tex}$.<br>"
    sol += "Gleichung für $\\lambda$: $|\\vec{OG}(\\lambda)| = #{r.to_tex}$. Mit dem TR:<br>"
    sol += "<tt>[$#{a.to_a.map{|x| x.to_tex}.join(",")}$]$\\rightarrow$ a<br>"
    sol += "[$#{b.to_a.map{|x| x.to_tex}.join(",")}$]$\\rightarrow$ b<br>"
    sol += "$#{r.to_tex} \\rightarrow$ r<br>"
    sol += "a+l*(b-a) $\\rightarrow$ g(l)</tt><br>"
    sol += "<tt>zeros(norm(g(l))-r, l) $\\rightarrow$ lx</tt><br>"
    sol += "<tt>g(lx[1])<br>g(lx[2])</tt><br>"
    sol += "Man erhält $\\lambda_1 = #{lambda[0].to_tex}$ und $\\lambda_2 = #{lambda[1].to_tex}$ und "
    sol += "die Schnittpunkte $S_1=#{t[0].to_tex(false)}$ und $S_2=#{t[1].to_tex(false)}$."
    return ex,sol
  end
end

ex = ARGV[0].to_i

n = 10

if (ex==1)

  trip = tripel(8)
  e = Array.new(n){gerade_kugel(trip)}
  
  # puts text
  name = "lineintersphere"
  res = "Bestimmen Sie eine Parameterdarstellung der Geraden $g$ durch die Pukte $A$ und $B$. Mit Hilfe des TR bestimmen Sie dann die Schnittpunkte von $g$ mit der Kugel mit Zentrum $(0,0,0)$ und Radius $r$.\n"
  res += header(name)
  res += e.inspect
  res += footer(name, " <br><hr> ", "<br><hr>")+"\n"

  xclip(res)
end
    
