# coding: utf-8
require './maxima'

class Rational
  def to_tex(amplify=1)
    if denominator==1
      return to_i.to_s
    else
      s = numerator<0 ? "-" : ""
      return "#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}"
    end
  end  
end

def gcd(a,b)
  return a if b==0
  return gcd(b, a%b)
end

def lcm(a,b)
  return a*b/gcd(a,b)
end

def gcds(a)
  if (a.size==2)
    return gcd(a[0],a[1])
  else
    return gcd(a[-1], gcds(a[0..-2]))
  end
end

def lcms(a)
  if  (a.size==2)
    return lcm(a[0],a[1])
  else
    return lcm(a[-1], lcms(a[0..-2]))
  end
end



def randFrac(min,max)
  r=nil
  20.times{|i|
    denom = rand(max-min+1)+min
    num = rand(max-min+1)+min
    r=Rational(num,denom)*(rand(2)*2-1)
    return r if r.denominator>=min and r.numerator>=min
  }
  return r
end

def lineareGleichungen(numEx=3)
  puts "Lösen Sie schrittweise auf:"
  max = Maxima.new
  exs = []
  sols = []
  variants=[]
  numEx.times{|i|
    while (true)
      x = randFrac(3,9)
      mul = randFrac(3,9)
      add = [randFrac(3,9), randFrac(3,9)]
      cd = lcms([mul+add[0], add[1], x*mul+add[1], add[0]].map{|r| r.denominator})
      max.command("g: x=#{x};");
      sol = max.extractTeX(max.command("tex(g);")).gsub("="," &= ")+"\n"
      max.command("g: g*#{mul.numerator};");
      sol = max.extractTeX(max.command("tex(g);")).gsub("="," &= ")+" && |:#{mul.numerator}\\\\\n"+sol
      max.command("g: g+#{add[0]*cd}*x+#{add[1]*cd};");
      sol = max.extractTeX(max.command("tex(g);")).gsub("="," &= ")+" && |-\\left(#{(add[0]*cd).to_i}x+#{(add[1]*cd).to_i}\\right)\\\\\n"+sol
      max.command("g: ratexpand(g/#{cd});");      
      ex = max.extractTeX(max.command("tex(g);"))
      sol = ex.gsub("="," &= ")+" && |\\cdot #{cd}\\\\\n"+sol
      
      unless (sol =~ /\d{3}/ || sol =~ /[7-9]\d/)
        variants.push(["$#{ex}$","$\\begin{align*}#{sol}\\end{align*}$"]);
        sols.push("#{i+1}.\n\n$\\begin{align*}\n#{sol}\\end{align*}$\n\n")
        #puts [x,mul,add,cd]
        #puts "***"
        break
      end
    end
  }  
  max.quit
  puts exs
  puts "\n\n<hidden Lösungen>"
  puts sols
  puts "</hidden>"
  p variants
end

n = ARGV[0] ? ARGV[0].to_i : 3
lineareGleichungen(n)


=begin

(%i4) g: [x=2,y=-3];
(%o4)                          [x = 2, y = - 3]
(%i5) g+5;
(%o5)                       [x + 5 = 7, y + 5 = 2]
(%i6) g+g[0];

apply: no such list element: [0]
 -- an error. To debug this try: debugmode(true);
(%i7) g[1]
;
(%o7)                                x = 2
(%i8) g+g[1];
(%o8)       [2 x = x + 2, y + x = x - 3] = [x + 2 = 4, y + 2 = - 1]
(%i9) g;
(%o9)                          [x = 2, y = - 3]
(%i10) g+(x=2);
(%o10)      [2 x = x + 2, y + x = x - 3] = [x + 2 = 4, y + 2 = - 1]
(%i11) g[1]+g[2];
(%o11)                            y + x = - 1
(%i12) 3*g[1]-2*g[2];
(%o12)                          3 x - 2 y = 12
(%i13) (3*g[1]-2*g[2])/6;
                                 3 x - 2 y
(%o13)                           --------- = 2
                                     6
(%i14) expand((3*g[1]-2*g[2])/6);
                                   x   y
(%o14)                             - - - = 2
                                   2   3
(%i15) 

=end
