# coding: utf-8
require './maxima'

class Rational
  def to_tex(amplify=1)
    if denominator==1
      return to_i.to_s
    else
      s = numerator<0 ? "-" : ""
      return "#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}"
    end
  end  
end

def gcd(a,b)
  return a if b==0
  return gcd(b, a%b)
end

def lcm(a,b)
  return a*b/gcd(a,b)
end

def gcds(a)
  if (a.size==2)
    return gcd(a[0],a[1])
  else
    return gcd(a[-1], gcds(a[0..-2]))
  end
end

def lcms(a)
  if  (a.size==2)
    return lcm(a[0],a[1])
  else
    return lcm(a[-1], lcms(a[0..-2]))
  end
end



def randFrac(min,max)
  r=nil
  20.times{|i|
    denom = rand(max-min+1)+min
    num = rand(max-min+1)+min
    r=Rational(num,denom)*(rand(2)*2-1)
    return r if r.denominator>=min and r.numerator>=min
  }
  return r
end

def non_colinear_rand_vecs(min=3, max=9)
  vecs = nil
  20.times {
    vecs = Array.new(2) {
      f = randFrac(3,9)
      [(rand(2)*2-1)*f.numerator, (rand(2)*2-1)*f.denominator]
    }
    if vecs[0][0]*vecs[1][1]-vecs[0][1]*vecs[1][0] != 0
      return vecs
    end
  }
  return vecs
end



def gleichungsSysteme(numEx=3)
  max = Maxima.new
  variants=[]
  numEx.times{|i|
    while (true)
      sols = Array.new(2) {(rand(2)*2-1)*(rand(4)+2)}
      sol = "$x=#{sols[0]}$ und $y=#{sols[1]}$"
      
      max.command("g: [x=#{sols[0]}, y=#{sols[1]}];")
      v = non_colinear_rand_vecs(3,7)
      # p v
      d = v.map{|a| a[0]*a[1]}
      max.command("g2: [expand((#{v[0][0]}*g[1]+#{v[0][1]}*g[2])/#{d[0]}), expand((#{v[1][0]}*g[1]+#{v[1][1]}*g[2])/#{d[1]})];")
      max.command("g3: [expand((#{v[0][0]}*g[1]+#{v[0][1]}*g[2])), expand((#{v[1][0]}*g[1]+#{v[1][1]}*g[2]))];")
      ex = "$\\left\\{\\begin{array}{rl} "
      sol1 = "$\\left\\{\\begin{array}{rll} "
      max.command("tex(g2[1]);")
      eqs = Array.new(2){|i|
        "\\displaystyle "+max.extractTeX(max.command("tex(g2[#{i+1}]);")).gsub("="," &\\displaystyle = ")
      }
      ex += eqs.join("\\\\\n")
      sol1 += eqs.map.with_index{|e,i| e+" \\quad & | \\cdot #{d[i]}"}.join("\\\\\n")    
      sol1 += "\\end{array}\\right. \\qquad \\Leftrightarrow \\qquad $"
      ex += "\\end{array}\\right.$\n"
      
      eqs = Array.new(2){|i|
        cmd = "tex(g3[#{i+1}]);";
        "\\displaystyle "+max.extractTeX(max.command(cmd)).gsub("="," &\\displaystyle = ")+" \\qquad & (#{i+1})"
      }
      sol1 += " &nbsp; $\\left\\{\\begin{array}{rll} "
      sol1 += eqs.join("\\\\\n");
      sol1 += "\\end{array}\\right.$\n"
      max.command("lx: solve(g3[1],x)[1];");
      sol1 += "<br>Gleichung (1) nach $x$ aufgelöst: $"+max.extractTeX(max.command("tex(lx);"))+"$ und in (2) eingesetzt:<br>";
      max.command("gy: subst(lx, g3[2]);");
      sol1 += "$"+max.extractTeX(max.command("tex(gy);"))+"$";
      max.command("gy : expand(gy);");
      sol1 += "$ \\qquad \\Leftrightarrow \\qquad "+ max.extractTeX(max.command("tex(gy);"))+"$";
      sol1 += " nach $y$ aufgelöst: $y=#{sols[1]}$<br>"
      # p max.command("tex(subst(y=#{sols[1]},lx));")
      sol1 += "Eingesetzt in $ "+ max.extractTeX(max.command("tex(lx);"))+ "=#{sols[0]}$<br>"
      
      sol = sol1+"Lösung: &nbsp; "+sol+"<br><hr>"
      
      # puts sol
      
      # puts "------"
      unless (sol=~/\d{3}/ || sol=~/[789]\d/)
        variants.push([ex,sol]);
        break
      end
    end
  }
  p variants
end


n = ARGV[0] ? ARGV[0].to_i : 3
gleichungsSysteme(n)


=begin

(%i4) g: [x=2,y=-3];
(%o4)                          [x = 2, y = - 3]
(%i5) g+5;
(%o5)                       [x + 5 = 7, y + 5 = 2]
(%i6) g+g[0];

apply: no such list element: [0]
 -- an error. To debug this try: debugmode(true);
(%i7) g[1]
;
(%o7)                                x = 2
(%i8) g+g[1];
(%o8)       [2 x = x + 2, y + x = x - 3] = [x + 2 = 4, y + 2 = - 1]
(%i9) g;
(%o9)                          [x = 2, y = - 3]
(%i10) g+(x=2);
(%o10)      [2 x = x + 2, y + x = x - 3] = [x + 2 = 4, y + 2 = - 1]
(%i11) g[1]+g[2];
(%o11)                            y + x = - 1
(%i12) 3*g[1]-2*g[2];
(%o12)                          3 x - 2 y = 12
(%i13) (3*g[1]-2*g[2])/6;
                                 3 x - 2 y
(%o13)                           --------- = 2
                                     6
(%i14) expand((3*g[1]-2*g[2])/6);
                                   x   y
(%o14)                             - - - = 2
                                   2   3
(%i15) 

=end
