def solve(f, a,b, eps=1.0e-7)
  while f.call(a).abs>eps
    #puts "a=#{a}, f(a)=#{f.call(a)}, b=#{b}, f(b)=#{f.call(b)}"
    c = (a+b)/2.0
    if f.call(a)*f.call(c)>0
      a=c
    else
      b=c
    end
  end
  return a
end


def zeros(f, xmin=-10.001, xmax=9.999, steps=1000)
  res = []
  dx = (xmax-xmin).to_f/steps
  steps.times{|i|
    x = i*dx+xmin
    #r = f.call(x)*f.call(x+dx)
    if f.call(x)*f.call(x+dx)<=0
      #puts "Look around x=#{x}"
      res.push(solve(f,x,x+dx))
    end
  }
  return res
end

def ddx(f, dx=1.0e-4)
  return lambda{|x| (f.call(x+dx)-f.call(x))/dx}
end

def nice(x)
  if (x-x.to_i).abs<1e-05
    return x.to_i
  end
  if (x-x.to_i-1).abs<1e-05
    return (x.to_i+1)
  end
  return x
end

def discuss(f, xmin=-10.0001, xmax=9.9999, runden=1)
  x = zeros(f, xmin, xmax)+zeros(ddx(f), xmin, xmax)+zeros(ddx(ddx(f)), xmin, xmax)
  x = x.map{|x| x.round(4)}.sort.uniq
  y = x.map{|x| f.call(x).round(runden)}
  m = x.map{|x| ddx(f).call(x).round(runden)}
  f2 = x.map{|x| ddx(ddx(f)).call(x).round(runden)}
  f3 = x.map{|x| ddx(ddx(ddx(f))).call(x).round(runden)}
  x = x.map{|x| x.round(runden)}
  table=[x,y,m,f2,f3].map{|z| z.map{|x| nice(x)}}
  heads = ["$x$", "$f(x)$", "$f'(x)$", "$f''(x)$", "$f'''(x)$"]
  tex = "\\begin{array}{l|"+"c"*x.size+"}\n"
  heads.each.with_index{|h,i|
    tex += h + " & " + table[i].join(" & ") + "\\\\\n"
    tex += "\\hline " if i==0
  }
  tex += "\\end{array}"
  puts tex
end



discuss(lambda{|x| 0.05*(x+2)*(x-2)*(x-2)*(x-4)})
