# coding: utf-8
require_relative "../header-footer.rb"


def solve(f, a,b, eps=1.0e-14)
  while b-a>eps
    #puts "a=#{a}, f(a)=#{f.call(a)}, b=#{b}, f(b)=#{f.call(b)}"
    c = (a+b)/2.0
    if f.call(a)*f.call(c)>0
      a=c
    else
      b=c
    end
  end
  return a
end


def zeros(f, xmin=-10.001, xmax=9.999, steps=1000)
  res = []
  dx = (xmax-xmin).to_f/steps
  steps.times{|i|
    x = i*dx+xmin
    #r = f.call(x)*f.call(x+dx)
    if f.call(x)*f.call(x+dx)<=0
      #puts "Look around x=#{x}"
      res.push(solve(f,x,x+dx))
    end
  }
  return res
end

def ddx(f, dx=1.0e-5)
  return lambda{|x| (f.call(x+dx)-f.call(x-dx))/dx/2.0}
end

def nice(x)
  if (x-x.to_i).abs<1e-05
    return x.to_i
  end
  if (x-x.to_i-1).abs<1e-05
    return (x.to_i+1)
  end
  return x
end

def discuss(f, numex, xmin=-10.0001, xmax=9.9999, runden=1, ftex=nil)
  x = zeros(f, xmin, xmax)+zeros(ddx(f), xmin, xmax)+zeros(ddx(ddx(f)), xmin, xmax)
  x = x.map{|x| x.round(4)}.sort.uniq
  y = x.map{|x| f.call(x).round(runden)}
  m = x.map{|x| ddx(f).call(x).round(runden)}
  f2 = x.map{|x| ddx(ddx(f)).call(x).round(runden)}
  #f3 = x.map{|x| ddx(ddx(ddx(f))).call(x).round(runden)}
  x = x.map{|x| x.round(runden)}
  table=[x,y,m,f2].map{|z| z.map{|x| nice(x)}}
  heads = ["x", "f(x)", "f'(x)", "f''(x)"]
  tex = "\\begin{array}{l|"+"c"*x.size+"}\n"
  heads.each.with_index{|h,i|
    tex += h + " & " + table[i].join(" & ") + "\\\\\n"
    tex += "\\hline " if i==0
  }
  tex += "\\end{array}"

  if ftex
    return ["$#{ftex}$", "$$#{tex}$$"]
  end
  puts tex

  minx = x.min.floor-1
  maxx = x.max.ceil+1
  miny = y.min.floor-1
  maxy = y.max.ceil+1

    asy = "import ksbg; 
    import graph; 
    size(8cm,0);
    koordinatenSystem(minp=(#{minx},#{miny}), maxp=(#{maxx}, #{maxy}),  // system size
                    lo=\"\", 
                    ticks=(1,1),
                    grid=(1,1),
                    lx=\"$x$\", ly=\"$y$\",  
                    ticksSize=0.04,  
                    axisoverhang=(0.2,0.2)
                    );
    "

    draw = (0...(x.size)).to_a.map{|i|
            "(#{x[i]},#{y[i]}){(1,#{m[i]})}"}

    xx = x[0]-(xmax-xmin)/20.0;
    draw.unshift("(#{xx},#{f.call(xx)}){(1,#{ddx(f).call(xx)})}")
    xx = x[-1]+(xmax-xmin)/20.0;
    draw.push("(#{xx},#{f.call(xx)}){(1,#{ddx(f).call(xx)})}")
    
    
    asy += "draw(#{draw.join("::")}, red);\n"
              
    asy += (0...(x.size)).to_a.map{|i|
            "draw(circle((#{x[i]},#{y[i]}),#{(maxx-minx)/100.0}));\n"          
            }.join("")
              
    asy += (0...(x.size)).to_a.map{|i|
          dx = 1.0
          dy = m[i].to_f
          if (dy.abs>1)
             dx = dx/dy.abs
             dy = dy/dy.abs
          end
          "draw((#{x[i]-dx},#{y[i]-dy})--(#{x[i]+dx},#{y[i]+dy}), green);\n"
        }.join("")
            
    
    x.size.times{|i|
        p table[i]
        if table[1][i]==0.0
          asy+="label(\"NS\", (#{x[i]},0), #{m[i]>0 ? "SE" : "NE"});"
        end
        if table[2][i]==0.0
           if f2[i]>0
              asy+="label(\"MIN\", (#{x[i]},#{y[i]}), S);"
           else
              asy+="label(\"MAX\", (#{x[i]},#{y[i]}), N);"
           end 
        end
        if (table[3][i]==0.0) 
           asy+="label(\"WS\", (#{x[i]},#{y[i]}), #{m[i]>0 ? "NW" : "SW"});"
        end
    }
    
    e=(xmax-xmin)/20.0
    asy += "clip((#{minx-e},#{miny-e})--(#{maxx+e},#{miny-e})--(#{maxx+e},#{maxy+e})--(#{minx-e},#{maxy+e})--cycle);"

    #puts asy
              
    asyfile = "kurvendiskussion-sol-#{numex}.asy"
    File.open(asyfile, "w") {|f|
        f.puts(asy)
    }
    
    pdf = asyfile.gsub(/asy$/,"pdf")
    png = asyfile.gsub(/asy$/,"png")
    
    `asy -f pdf #{asyfile}`
    `convert -alpha Off -density 300x300 #{pdf} #{png}`
    `rm #{pdf}`
    return "$$#{tex}$$", "<HTML><img src='https://fginfo.ksbg.ch/~ivo/miniaufgaben/#{png}'></HTML>\n"
end

[lambda{|x| 4*(1.0/(1.0+x*x)-0.2)},
 lambda{|x| 0.5*(x+1)*(x-1)*(x-2)},
 lambda{|x| 0.4*(x+2)*(x-1)*x*x},
 ]

exsol= nil
name = "kurvendiskussion_tabelle_graph"
res = "Skizzieren Sie den Graphen. Zeichnen Sie zuerst die Tangenten ein, und beschriften Sie die Punkte (NS, MIN, MAX, WS). Es darf davon ausgangen werden, dass in allen Punkten $f'''(x)\\neq0$.\n"

if ARGV[0].to_i==2
  name = "kurvendiskussion_funktion_tabelle"
  res = "**Mit Hilfe des Taschenrechners** erstellen Sie innerhalb von **5 Minuten** eine Tabelle mit allen Nullstellen, Kandidaten für Extremal- und Wendestellen sowie die zugehörigen Werte der folgenden Funktion, der ersten und zweiten Ableitung. Runden Sie die Resultate auf 2 Nachkommastellen.\n"
  exsol = [[lambda{|x| (x-1)*(x-1)*(x+1)*(x+1)},"f(x)=(x+1)^2\\cdot(x-1)^2"],
           [lambda{|x| (1-x)*(x-1)*(x+1)*(x+1)},"f(x)=(x-1)\\cdot(1-x)\\cdot(x+1)^2"],
           [lambda{|x| (x-2)*(x-2)*(x+2)*(x+2)},"f(x)=(x+2)^2\\cdot(x-2)^2"],
           [lambda{|x| (2-x)*(x-2)*(x+2)*(x+2)},"f(x)=(x-2)\\cdot(2-x)\\cdot(x+2)^2"],
           [lambda{|x| (x+1)*(x+1)*(x-2)},"f(x)=(x+1)^2\\cdot(x-2)"],
           [lambda{|x| (x+1)*(x+1)*(2-x)},"f(x)=(x+1)^2\\cdot(2-x)"],
           [lambda{|x| (x+1)*(x-2)*(x-2)},"f(x)=(x+1)\\cdot(x-2)^2"],
           [lambda{|x| (x+1)*(x-2)*(2-x)},"f(x)=(x+1)\\cdot(2-x)\\cdot(x-2)"],
           [lambda{|x| (x+1)**3*(x-1)},"f(x)=(x+1)^3\\cdot(x-1)"],
           [lambda{|x| (x+1)**3*(1-x)},"f(x)=(x+1)^3\\cdot(1-x)"],
          ].map.with_index{|f,i|
    ex,sol = discuss(f[0],i,-10.0001, xmax=9.9999, runden=2, ftex=f[1])
    [ex,sol]\
  }
else
  exsol = [lambda{|x| 4*(1.0/(1.0+x*x)-0.2)+0.5*x},
           lambda{|x| 8.0/(1+Math.exp(x))-2+x},
           lambda{|x| 0.5*(x+1)*(x-2)*(x-2)},
           lambda{|x| -0.5*(x+2)*(x+2)*(x-1)},
           #
           lambda{|x| 4*(-1.0/(1.0+x*x)+0.2)+0.5*x},
           lambda{|x| 8.0/(1+Math.exp(-x))-2-x},
           lambda{|x| 0.5*(x+1)*x*(x-2)-1},
           lambda{|x| -0.5*(x+2)*(x+2)*(x-1)-1},
           #
           lambda{|x| (x+1)*(x+1)+1.0/(1.0+Math.exp(7*x))-2.5},
           lambda{|x| -((x-1)*(x-1)+1.0/(1.0+Math.exp(-7*x)))+2.5},
          ].map.with_index{|f,i|
    ex,sol = discuss(f,i)
    [ex,sol]
  }
end


res += header(name)+exsol.inspect+footer(name)
xclip(res)

