# coding: utf-8
require "./header-footer.rb"

class Rational
  def to_tex(op=false, amplify=1)
    if denominator==1
      return (op && self>=0 ? "+" : "")+to_i.to_s
    else
      s = numerator<0 ? "-" : (op ? "+" : "")
      return "#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}"
    end
  end  
end

def ncr(n,m)
  r = 1
  for x in (n-m+1)..n
    r*=x
  end
  for x in 2..m
    r/=x
  end
  return r
end

def meanmedian()
  ex = ""
  sol = ""
  nr = "a";
  for n in 5..6
    mu = rand(10)+10
    w = Array.new(n-1){mu+rand(10)-rand(10)}
    w += [n*mu-w.sum]
    med = Rational(w.sort[n/2],1)
    if (n%2==0) 
      med = (med+w.sort[n/2-1])/2
    end 
    ex += "#{nr}) "+w.join(", ")+" &nbsp; "
    sol += "#{nr}) $\\mu = #{mu}$, $\\tilde x = #{med.to_tex}"
    if (med.denominator>1)
      sol+= " = #{med.to_f}"
    end
    sol+= "$ &nbsp; "
    nr.succ!
  end
  return [ex,sol]
end

def stdev()
  while true
    mu = rand(10)+10
    n=5
    w = Array.new(n-1){mu+rand(7)-rand(7)}
    w += [n*mu-w.sum]
    abw = w.map{|x| (x-mu)*(x-mu)}
    next if abw.uniq.size<4
    q = abw.sum
    # next if q%(n-1)!=0
    # q/=(n-1)
    next if q<8
    sq = Math.sqrt(q).to_i
    if q==sq*sq
      ex = w.join(", ")
      sol = "$\\mu = #{mu}$, $\\sigma^2 = \\frac{1}{n-1} \\cdot \\sum_{i=1}^{n} (x_i-\\mu)^2 = "
      sol += "\\frac{1}{#{n-1}} \\cdot \\left(#{abw.join("+")}\\right) "
      q = Rational(q,(n-1))
      sol += " = \\frac{1}{#{n-1}}\\cdot #{abw.sum} = #{q.to_tex}$ also "
      sol += "$\\sigma = \\sqrt{#{q.to_tex}}"
      if (Math.sqrt(n-1)==Math.sqrt(n-1).to_i)
        sq= Rational(sq,Math.sqrt(n-1).to_i)
        sol += " = #{sq.to_tex()}"
      end
      sol+="$"
      return [ex,sol]
    end
  end
end

def quartile() 
    while true
        n = 40+2*rand(30);
        w = Array.new(n) {rand(100)+rand(100)}.sort
        q1 = (n-1)/4-rand(4)
        q3 = 3*(n-1)/4-rand(4);
        ex = "Anzahl Werte $n=#{n}$.<br>$\\begin{array}{lccccclcccccl}\n"
        ex += "\\ldots"
        5.times{|i|
            ex += " & x_{#{q1+i+1}}"
        }
        ex += " & \\ldots "
        5.times{|i|
            ex += " & x_{#{q3+i+1}}"           
        }
        ex += "\\\\\n \\ldots"
        5.times{|i|
            ex += " & #{w[q1+i]}"
        }
        ex += " & \\ldots"
        5.times{|i|
            ex += " & #{w[q3+i]},"           
        }
        ex += "\\\\\n \\end{array}$"
        q1 = (n-1)/4
        q3 = (n-1)*3/4
        
        next if w[q1]==w[q1+1] || w[q3]==w[q3+1]
        ex += "$\\ldots$"
        t = 0.25*(n-1)
        t -= t.to_i
        q25 = (1-t)*w[q1]+t*w[q1+1];
        sol = "Position für erstes Quartil: $1+0.25\\cdot #{n-1} = #{0.25*(n-1)+1}$. "
        sol += "Wir suchen also einen Wert an der Stelle #{t} zwischen $x_{#{q1+1}}=#{w[q1]}$ und $x_{#{q1+2}}=#{w[q1+1]}$. Das erste Quartil ist damit "
        sol += "$#{w[q1]} + #{t} \\cdot (#{w[q1+1]}-#{w[q1]}) = #{q25}$<br>"

        t = 0.75*(n-1)
        t -= t.to_i
        q75 = (1-t)*w[q3]+t*w[q3+1];
        sol += "Position für drittes Quartil: $i=1+0.75\\cdot #{n-1} = #{0.75*(n-1)+1}$. "
        sol += "Wir suchen also einen Wert an der Stelle #{t} zwischen $x_{#{q3+1}}=#{w[q3]}$ und $x_{#{q3+2}}=#{w[q3+1]}$. Das dritte Quartil ist damit "
        sol += "$#{w[q3]} + #{t} \\cdot (#{w[q3+1]}-#{w[q3]}) = #{q75}$<br>"
        
        return [ex,sol]
    end 
end

def umfrage()
    wurzel_n = [10,12,15,20,25,30,40,50]
    return wurzel_n.map{|w|
        n = w*w
        mean = 0
        stdev = 0
        while true
            mean = rand(5)+3
            stdev = mean*(10-mean)
            next if w%3==0 && stdev%3!=0
            break    
        end
        mean*=0.1
        stdev*=0.01
        stdevm = stdev/w
        ex = "$n=#{n}$, $\\mu=#{"%.2f" % mean}$, $\\sigma=#{"%.2f" % stdev}$."
        sol = "$\\sigma_{\\mu} = \\frac{\\sigma}{\\sqrt{n}} = \\frac{#{"%.2f" % stdev}}{#{w}} = #{"%.4f" %stdevm}$<br>"
        sol += "95%-Vertrauensintervall: $\\mu \\pm 2\\sigma_{\\mu}$ also zwischen #{"%.2f" % (100*(mean-2*stdevm))}% und #{"%.2f" % (100*(mean+2*stdevm))}% Anteil Ja-Antworten."
        [ex,sol]
    }
end

  
type = ARGV[0]
res = "" 
if (type.to_i==1)
  name = "meanmed"

  all = Array.new(10){meanmedian()}
  
  res = "Berechnen Sie den Mittelwert $\\mu$ und den Median $\\tilde x$ für die folgenden zwei Wertereihen.\n"
  res += header(name)
  res += all.inspect
  res += footer(name, " <hr> ", " <hr> ")+"\n"
elsif (type.to_i==2)
  name = "standardabweichung"

  all = Array.new(10){stdev()}
  
  res = "Berechnen Sie Standardabweichung der folgenden Wertereihe.\n"
  res += header(name)
  res += all.inspect
  res += footer(name, " <hr> ", " <hr> ")+"\n"
elsif (type.to_i==3)
  name = "vertrauensintervall"

  
  res = "In einer Umfrage wird etwas gefragt, worauf nur die Antwort Nein(0) oder Ja(1) gegeben wird. Die Anzahl $n$ der Antworten, der Durchschnitt $\\mu$ der Antworten und die Standardabweichung $\\sigma$ der Wertereihe sind bekannt. Geben Sie 95%-Vertrauensintervall für das Umfrageergebnis an.\n"
  res += header(name)
  res += umfrage().inspect
  res += footer(name, " <hr> ", " <hr> ")+"\n"
elsif (type.to_i==4)
  name = "Quartile"

  
  res = "Gegeben ist eine aufsteigend sortierte Wertereihe $x_1$, $x_2$, $\\ldots$, $x_n$. Berechnen Sie das erste und dritte Quartil folgender Wertereihe:\n"
  res += header(name)
  res += Array.new(10) {quartile()}.inspect
  res += footer(name, " <hr> ", " <hr> ")+"\n"
end

xclip(res)
