# coding: utf-8

require './header-footer.rb'


class Rational
  def to_tex(amplify=1, paren=false)
    p=(paren && (self<0)) ? ["\\left(", "\\right)"] : ["",""]
    if denominator==1
      return p[0]+numerator.to_s+p[1]
    else
      s = numerator<0 ? "-" : ""
      return "#{p[0]}#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}#{p[1]}"
    end
  end  
end

def randRat(nmin, nmax, dmin, dmax)
  r = nil
  50.times{
    r = Rational(rand(nmax+1)+nmin, rand(dmax+1)+dmin)
    if r.denominator>=dmin && r.numerator>=nmin
      r*=(rand(2)*2-1)
      return r
    end
  }
  return r
end



class Monome
  attr_accessor :coeff, :varexps
  def initialize(copy=nil)
    if copy.is_a?(Monome)
      @coeff=copy.coeff
      @varexps = Array.new(copy.varexps.size){|i|
        Array.new(copy.varexps[i])
      }
    elsif copy.is_a?(Numeric)
      @coeff = Rational(copy)
      @varexps = []
    elsif copy.is_a?(String)
      @coeff = Rational(1)
      @varexps=[[String.new(copy), 1]]
    else
      @coeff = Rational(0)
      @varexps = []
    end
  end

  def copy()
    return Monome.new(self)
  end
  
  def collect!
    #Find duplicates
    pos = 0
    while pos<@varexps.size
      v = @varexps[pos][0]
      p2 = pos+1
      while p2<@varexps.size
        if varexps[p2][0]==v
          @varexps[pos][1]+=@varexps[p2][1]
          @varexps.delete_at(p2)
        else
          p2+=1
        end
      end
      pos+=1
    end
    return self
  end

  def name
    self.copy().order!().collect!().varexps.map{|ve| "#{ve[0]}^{#{ve[1]}}"}.join("")
  end

  def deg
    @varexps.inject(0){|s,ve| s+ve[1]}
  end
  
  def collect
    return copy().collect!()
  end
  
  def order!
    @varexps.sort!{|a,b| a[0] <=> b[0]}
    return self
  end

  def order()
    return self.copy().order!()
  end
  
  def to_tex(minusparen=false, plus=false)
    pars = [["",""],["\\left(","\\right)"]][minusparen && @coeff<0 ? 1 : 0]
    sign = @coeff<0 ? "-" : (plus ? "+" : "")
    name = @varexps.map{|x| "#{x[0]}#{x[1]>1 ? "^"+x[1].to_s : ""}"}.join("")
    c = @coeff.abs==1 ? "" : @coeff.abs.to_tex()
    return pars[0]+sign+c+name+pars[1]      
  end
  
  def *(m)
    if (m.is_a?(Numeric))
      res = self.copy()
      res.coeff *= m
      return res
    elsif (m.is_a?(Monome))
      res = Monome.new(self)
      res.coeff *= m.coeff
      res.varexps+=m.varexps
      return res
    elsif (m.is_a?(Polynome))
      return m*self
    else
      raise "Sorry, in Monome::multiply, don't know what to do about #{m} of class #{m.class}"
    end
  end
  
  def +(m)
    return self.to_poly()+m            
  end
  
  def -(m)
    return self.to_poly()+m*(-1)
  end

  def **(i)
    unless i.is_a?(Integer) && i>=0
      raise "Can only raise to the power of a positive Integer"
    end
    if i==0
      return Monome.new(1)
    end
    if i==1
      return copy()
    end
    res = copy()
    res.varexps.map!{|ve| [ve[0], ve[1]*i]}
    return res     
  end
  
  def to_poly()
    p = Polynome.new()
    p.monomes = [Monome.new(self)]
    return p
  end
end

class Polynome
  attr_accessor :monomes
  def initialize(copy = nil)
    if (copy)
      @monomes = Array.new(copy.monomes.size){|i|
        copy.monomes[i].copy()
      }
    else
      @monomes = [Monome.new()]
    end 
  end
  
  def to_tex()
    return @monomes.map.with_index{|m,i| m.to_tex(false, i>0)}.join("")
  end
  
  def *(p)
    if (p.is_a?(Numeric) || p.is_a?(Monome))
      res = Polynome.new(self)
      res.monomes.map!{|m| m*p}
      return res
    elsif (p.is_a?(Polynome))
      res = Polynome.new
      res.monomes = []
      @monomes.each{|m1|
        p.monomes.each{|m2|
          res.monomes.push(m1*m2)
        }
      }
      return res
    else       
      raise "Sorry, in Polynome::multiply, don't know what to do about #{p} of class #{p.class}"              
    end  
  end

  def copy
    return Polynome.new(self)
  end
  
  def collect!
    @monomes.map!{|m| m.order!().collect!()}
    return self
  end

  def collect
    return self.copy().collect!()
  end

  def order!()
    @monomes.sort!{|a,b| a.deg==b.deg ? a.name <=> b.name : b.deg <=> a.deg}
    return self
  end

  def order
    return self.copy().order!()
  end

  def sumUp!
    names = @monomes.map{|m| m.name}
    delete = []
    for i in 0...(@monomes.size-1)
      unless delete.include?(i)
        for j in (i+1)...(@monomes.size)
          if (names[j]==names[i])
            @monomes[i].coeff+=@monomes[j].coeff
            delete.push(j)
          end
        end
      end
    end
    delete.reverse.each{|j| @monomes.delete_at(j)}
    @monomes.select!{|m| m.coeff!=0}
    return self
  end

  def sumUp()
    return self.copy().sumUp!()
  end
  
  def +(p)
    if p.is_a?(Numeric)
      p = Monome.new(p)
    end
    if p.is_a?(Monome)
      res = Polynome.new(self)
      res.monomes.push(Monome.new(p))
      return res
    elsif p.is_a?(Polynome)
      res = Polynome.new(self)
      res.monomes+=p.monomes
      return res
    end
    raise "Sorry, in Polynome::add, don't know what to do about #{p} of class #{p.class}"
  end

  def -(p)
    return self+p*(-1)
  end
end 

def randPoly(var, terms, minexp, maxexp)
  res = Polynome.new()
  (minexp..maxexp).to_a.shuffle[0...terms].each{|e|
    # puts "randPoly var=#{var.inspect}"
    res+=(var**e)*randRat(2,6,2,8)
  }
  return res.order!().sumUp!()
end

def makeex
  a = Monome.new("a")
  
  while true
    p=randPoly(a,2,0,2)
    # p a
    q=randPoly(a,2,0,2)
    # p a
    e=randRat(2,6,2,8)
    f=randRat(2,6,2,8)
    ex = "\\left(#{p.to_tex}\\right) \\cdot #{e.to_tex(1,true)} \\cdot \\left(#{q.to_tex()}\\right) \\cdot #{f.to_tex(1,true)}"
    sol = "\\left(#{p.to_tex}\\right) \\cdot \\left(#{q.to_tex()}\\right) \\cdot #{e.to_tex(1,true)} \\cdot #{f.to_tex(1,true)}"
    r = p * q
    r.collect!()
    f = e*f
    next if f.denominator<2 || f.numerator.abs<2
    sol += " = \\left(#{r.to_tex}\\right) \\cdot #{f.to_tex(1,true)}"
    r.collect!().sumUp!().order!()
    next if r.monomes.size>3
    sol += " = \\left(#{r.to_tex}\\right) \\cdot #{f.to_tex(1,true)}"
    next if r.monomes.any?{|m| m.coeff.denominator>100 || m.coeff.numerator.abs > 100}
    r *= f
    next if r.monomes.any?{|m| m.coeff.denominator>100 || m.coeff.numerator.abs > 100}
    sol += " = "+ r.to_tex()
    
    #p ex
    #p sol
    #puts "$#{sol}$"
    return ["$#{ex}$", "$#{sol}$"]
  end
end
  
name = "multiausmult"
res = "Multiplizieren Sie vollständig aus, fassen Sie zusammen und geben Sie alle Brüche vollständig gekürzt an:\n"
res += header(name)
res += Array.new(10){makeex}.inspect
res += footer(name, " <br> ")+"\n"
xclip(res)
