#!/usr/bin/ruby
# coding: utf-8

require './header-footer.rb'
require "matrix"

class Rational
  def to_tex(amplify=1, paren=false)
    p=(paren && (self<0)) ? ["\\left(", "\\right)"] : ["",""]
    if denominator==1
      return p[0]+numerator.to_s+p[1]
    else
      s = numerator<0 ? "-" : ""
      return "#{p[0]}#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}#{p[1]}"
    end
  end  
end

class Vector
  def to_tex(vec=true)
    return "\\begin{pmatrix} "+self.map{|x| x.to_tex()}.to_a.join(" \\\\ ")+"\\end{pmatrix} " if vec
    return "\\left("+self.map{|x| x.to_tex()}.to_a.join(",\\,")+"\\right)"
  end
end


def tex2rat(s)
  s=s.gsub("\\cdot","*").gsub(":","/").gsub("\\left", "").gsub("\\right","").gsub("$","")
  s=s.gsub(/\\frac\{(.*?)\}\{(.*?)\}/,"Rational(\\1,\\2)").gsub("=",",")
  s
end

def gcd(a,b)
  return a if b==0
  return gcd(b, a%b)
end

def lcm(a,b)
  return a*b/gcd(a,b)
end

def gcds(a)
  if (a.size==2)
    return gcd(a[0],a[1])
  else
    return gcd(a[-1], gcds(a[0..-2]))
  end
end

def lcms(a)
  if  (a.size==2)
    return lcm(a[0],a[1])
  else
    return lcm(a[-1], lcms(a[0..-2]))
  end
end

def sum2tex(a,b,pm)
  cd = a.denominator*b.denominator/gcd(a.denominator,b.denominator)
  return a.to_tex(cd/a.denominator)+%w(- +)[pm]+b.to_tex(cd/b.denominator)
end

def randRat(nmin, nmax, dmin, dmax)
  r = nil
  50.times{
    r = Rational(rand(nmax+1)+nmin, rand(dmax+1)+dmin)
    if r.denominator>=dmin && r.numerator>=nmin
      r*=(rand(2)*2-1)
      return r
    end
  }
  return r
end

def hasSquare?(i)
   r=2;
   while (r*r<=i)
       if i % (r*r) == 0
           return true
       end
       r+=1
   end
   return false
end

def wurzel(i)
   c = 1
   r=2
   while (r*r<=i)
       if (i% (r*r) == 0)
          c*=r
          i/=(r*r)
       else
           r+=1
       end
   end
   return c,i       
end


def geradenex()
  while true
    g = Array.new(2) { Vector.elements(Array.new(3){Rational((rand(5)+2)*(rand(2)*2-1),1)}) }
    v = g[1]-g[0]
    next if v.any?{|x| x==0}
    lambda = Array.new(2) {Rational((rand(5)+2)*(rand(2)*2-1),1)}
    next if lambda[0]==lambda[1]
    p = lambda.map{|l| g[0]+v*l}
    offi = rand(2)
    offc = rand(3)
    a = p[offi].to_a
    a[offc] += rand(2)*2-1
    p[offi] = Vector.elements(a)
    perm = [0,1].shuffle
    ex = "$A=#{g[0].to_tex(false)},\\, B=#{g[1].to_tex(false)},\\,C=#{p[0].to_tex(false)},\\,D=#{p[1].to_tex(false)}$"
    sol = ex+"<br>\nParameterdarstellung: $\\vec{OG}(\\lambda) = \\vec{OA}+\\lambda \\cdot \\vec{AB} = #{g[0].to_tex()} + \\lambda #{v.to_tex}$\n<br>\n"
    %w(C D).each.with_index{|l,i|
        komp = [0,1,2]
        komp.delete(offc) if i==offi
        komp = komp.shuffle[0]
        sol += "Punkt $#{l}$, #{komp+1}. Komponente: $#{g[0][komp].to_tex}+\\lambda \\cdot #{v[komp].to_tex(1,true)} = #{p[i][komp].to_tex}$ liefert $\\lambda = #{lambda[i].to_tex}$. "
        sol += "Eingesetzt: $\\vec{OG}\\left(#{lambda[i].to_tex}\\right) = #{(g[0]+lambda[i]*v).to_tex}$. "
        if i==offi
            sol += "Also liegt $#{l}$ <b>nicht</b> auf $g$.\n<br>\n"
        else
            sol+="Also liegt $#{l}$ auf $g$.\n<br>\n"
        end
    }
    return ex,sol
  end
end

ex = ARGV[0].to_i

n = 10

if (ex==1)

  
  e = Array.new(n){geradenex()}
  
  # puts text
  name = "pointonline"
  res = "Geben Sie eine Parameterdarstellung der Geraden durch die Punkte $A$ und $B$ an. Bestimmen Sie dann, ob die Punkte $C$ und $D$ auf der Geraden durch die Punkte $A$ und $B$ liegen.\n"
  res += header(name)
  res += e.inspect
  res += footer(name, " <br><hr> ", "<br><hr>")+"\n"

  xclip(res)
elsif ex==2
  puts Array.new(2){geradenex()}
end
