#!/usr/bin/ruby
# coding: utf-8

require './header-footer.rb'
require "matrix"

class Rational
  def to_tex(amplify=1, paren=false)
    p=(paren && (self<0)) ? ["\\left(", "\\right)"] : ["",""]
    if denominator==1
      return p[0]+numerator.to_s+p[1]
    else
      s = numerator<0 ? "-" : ""
      return "#{p[0]}#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}#{p[1]}"
    end
  end  
end

class Vector
  def to_tex(vec=true)
    return "\\begin{pmatrix} "+self.map{|x| x.to_tex()}.to_a.join(" \\\\ ")+"\\end{pmatrix} " if vec
    return "\\left("+self.map{|x| x.to_tex()}.to_a.join(",\\,")+"\\right)"
  end
end


def tex2rat(s)
  s=s.gsub("\\cdot","*").gsub(":","/").gsub("\\left", "").gsub("\\right","").gsub("$","")
  s=s.gsub(/\\frac\{(.*?)\}\{(.*?)\}/,"Rational(\\1,\\2)").gsub("=",",")
  s
end

def gcd(a,b)
  return a if b==0
  return gcd(b, a%b)
end

def lcm(a,b)
  return a*b/gcd(a,b)
end

def gcds(a)
  if (a.size==2)
    return gcd(a[0],a[1])
  else
    return gcd(a[-1], gcds(a[0..-2]))
  end
end

def lcms(a)
  if  (a.size==2)
    return lcm(a[0],a[1])
  else
    return lcm(a[-1], lcms(a[0..-2]))
  end
end

def sum2tex(a,b,pm)
  cd = a.denominator*b.denominator/gcd(a.denominator,b.denominator)
  return a.to_tex(cd/a.denominator)+%w(- +)[pm]+b.to_tex(cd/b.denominator)
end

def randRat(nmin, nmax, dmin, dmax)
  r = nil
  50.times{
    r = Rational(rand(nmax+1)+nmin, rand(dmax+1)+dmin)
    if r.denominator>=dmin && r.numerator>=nmin
      r*=(rand(2)*2-1)
      return r
    end
  }
  return r
end

def hasSquare?(i)
   r=2;
   while (r*r<=i)
       if i % (r*r) == 0
           return true
       end
       r+=1
   end
   return false
end

def wurzel(i)
   c = 1
   r=2
   while (r*r<=i)
       if (i% (r*r) == 0)
          c*=r
          i/=(r*r)
       else
           r+=1
       end
   end
   return c,i       
end

def tripel(max)
    res = []
    for x in 1..max
        for y in x..max
            for z in y..max
                l2 = x*x+y*y+z*z
                if l2 == (Math.sqrt(l2).to_i)**2
                    res.push(Vector.elements([Rational(x,1),Rational(y,1),Rational(z,1)]))
                end
            end
        end
    end
    return res
end


def vecProd
    while true
      v = Array.new(2){Vector.elements(Array.new(3){randRat(2,5,3,8)})}
      negs = v.map{|vv| vv.count{|x| x<0}}.sum
      next if negs<2 || negs>3
      r = v[0].cross(v[1])
      next if r.any?{|x| x.denominator>15} || r.any?{|x| x.numerator.abs>25}
      next if r.any?{|x| x.denominator==1}

      next if Array.new(3){|i| (v[0][(i+1)%2]*v[1][(i+2)%2])}.any?{|x| x.denominator>100}
      next if Array.new(3){|i| v[0][(i+2)%2]*v[1][(i+1)%2]}.any?{|x| x.denominator>100}
      
      next if Array.new(3){|i| lcm(v[0][(i+1)%2].denominator, v[1][(i+2)%2].denominator)}.any?{|x| x>144}
      next if Array.new(3){|i| lcm(v[0][(i+2)%2].denominator, v[1][(i+1)%2].denominator)}.any?{|x| x>144}
            
      ex = v.map{|x| x.to_tex}.join(" \\times ")

      p ex
      p r
      
      sol = ex+" = \\begin{pmatrix}" + Array.new(3){|i|
        v[0][(i+1)%3].to_tex+" \\cdot "+v[1][(i+2)%3].to_tex(1,true)+" & - & "+v[0][(i+2)%3].to_tex(1,true)+" \\cdot "+v[1][(i+1)%3].to_tex(1,true)
      }.join(" \\\\ ")+"\\end{pmatrix} = "
      sol += "\\begin{pmatrix}" + Array.new(3){|i|
        (v[0][(i+1)%3]*v[1][(i+2)%3]).to_tex()+" & - &  "+((v[0][(i+2)%3]*v[1][(i+1)%3]).to_tex(1,true))
      }.join(" \\\\ ")+"\\end{pmatrix} = "
      sol += r.to_tex
      
      
      return ["$#{ex}$","$#{sol}$"]
    end
end


n = 10

e = Array.new(n){vecProd()}
                            
# puts text
name = "vectolength"
res = "Berechnen Sie von Hand. Im Resultat sind alle Brüche gekürzt.\n"
res += header(name)
res += e.inspect
res += footer(name, " <br><hr> ", "<br><hr>")+"\n"

xclip(res)

