#!/usr/bin/ruby
# coding: utf-8

require './header-footer.rb'
require "matrix"

class Rational
  def to_tex(amplify=1, paren=false)
    p=(paren && (self<0)) ? ["\\left(", "\\right)"] : ["",""]
    if denominator==1
      return p[0]+numerator.to_s+p[1]
    else
      s = numerator<0 ? "-" : ""
      return "#{p[0]}#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}#{p[1]}"
    end
  end  
end

class Vector
  def to_tex(vec=true)
    return "\\begin{pmatrix} "+self.map{|x| x.to_tex()}.to_a.join(" \\\\ ")+"\\end{pmatrix} " if vec
    return "\\left("+self.map{|x| x.to_tex()}.to_a.join(",\\,")+"\\right)"
  end
end


def tex2rat(s)
  s=s.gsub("\\cdot","*").gsub(":","/").gsub("\\left", "").gsub("\\right","").gsub("$","")
  s=s.gsub(/\\frac\{(.*?)\}\{(.*?)\}/,"Rational(\\1,\\2)").gsub("=",",")
  s
end

def gcd(a,b)
  return a if b==0
  return gcd(b, a%b)
end

def lcm(a,b)
  return a*b/gcd(a,b)
end

def gcds(a)
  if (a.size==2)
    return gcd(a[0],a[1])
  else
    return gcd(a[-1], gcds(a[0..-2]))
  end
end

def lcms(a)
  if  (a.size==2)
    return lcm(a[0],a[1])
  else
    return lcm(a[-1], lcms(a[0..-2]))
  end
end

def sum2tex(a,b,pm)
  cd = a.denominator*b.denominator/gcd(a.denominator,b.denominator)
  return a.to_tex(cd/a.denominator)+%w(- +)[pm]+b.to_tex(cd/b.denominator)
end

def randRat(nmin, nmax, dmin, dmax)
  r = nil
  50.times{
    r = Rational(rand(nmax+1)+nmin, rand(dmax+1)+dmin)
    if r.denominator>=dmin && r.numerator>=nmin
      r*=(rand(2)*2-1)
      return r
    end
  }
  return r
end

def hasSquare?(i)
   r=2;
   while (r*r<=i)
       if i % (r*r) == 0
           return true
       end
       r+=1
   end
   return false
end

def wurzel(i)
   c = 1
   r=2
   while (r*r<=i)
       if (i% (r*r) == 0)
          c*=r
          i/=(r*r)
       else
           r+=1
       end
   end
   return c,i       
end

def tripel(max)
    res = []
    for x in 1..max
        for y in x..max
            for z in y..max
                l2 = x*x+y*y+z*z
                if l2 == (Math.sqrt(l2).to_i)**2
                    res.push(Vector.elements([Rational(x,1),Rational(y,1),Rational(z,1)]))
                end
            end
        end
    end
    return res
end


def vec_to_len(trip)
    while true
        v = trip[rand(trip.size)]
        v = Vector.elements(v.to_a.map{|x| x*(rand(2)*2-1)})
        p v
        l = Rational(v.norm().to_i,1)
        nextr if l>12
        p l
        orig = randRat(2,6,2,5).abs
        target = randRat(2,6,2,5).abs
        factor = target/orig
        next if factor.denominator==1 || factor.numerator==1 || factor.denominator>10 || factor.numerator>10
        exv = v*orig
        solv = v*target
        p exv
        p solv
        squares = exv.to_a.map{|x| x*x}
        sqsum = squares.sum
        next if sqsum.denominator==1 || sqsum.denominator>144 || sqsum.numerator>144
        next if squares.any?{|x| x.denominator>144 || x.numerator>144}
        ex = "$#{exv.to_tex()}$ auf Länge $#{(target*l).to_tex()}$ skalieren."
        sol = "$\\left|#{exv.to_tex()}\\right| = \\sqrt{#{squares.map{|x| x.to_tex}.join("+")}} = \\sqrt{#{sqsum.to_tex}} =  #{(l*orig).to_tex}$. "
        sol += "Der Vektor muss also durch $#{(l*orig).to_tex}$ dividiert werden, um auf die Länge 1 skaliert zu werden, und danach mit der gewünschten Länge $#{(target*l).to_tex}$ multipliziert werden. "
        sol += "Man erhält $#{exv.to_tex()} \\cdot #{(Rational(1,1)/(l*orig)).to_tex} \\cdot #{(target*l).to_tex} = "
        sol += "#{exv.to_tex()} \\cdot #{factor.to_tex} = #{solv.to_tex}$."
        return ex,sol
    end
end

ex = ARGV[0].to_i

n = 10

if (ex==1)

  trip = tripel(8)
  e = Array.new(n){vec_to_len(trip)}
  
  # puts text
  name = "vectolength"
  res = "Skalieren Sie den Vektor auf die gewünschte Länge. Alle Brüche sind vollständig gekürzt anzugeben.\n"
  res += header(name)
  res += e.inspect
  res += footer(name, " <br><hr> ", "<br><hr>")+"\n"

  xclip(res)
end
    
