#!/usr/bin/ruby
# coding: utf-8

require './header-footer.rb'

class Rational
  def to_tex(amplify=1, paren=false)
    p=(paren && (self<0)) ? ["\\left(", "\\right)"] : ["",""]
    if denominator==1
      return p[0]+numerator.to_s+p[1]
    else
      s = numerator<0 ? "-" : ""
      return "#{p[0]}#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}#{p[1]}"
    end
  end  
end

def tex2rat(s)
  s=s.gsub("\\cdot","*").gsub(":","/").gsub("\\left", "").gsub("\\right","").gsub("$","")
  s=s.gsub(/\\frac\{(.*?)\}\{(.*?)\}/,"Rational(\\1,\\2)").gsub("=",",")
  s
end

def gcd(a,b)
  return a if b==0
  return gcd(b, a%b)
end

def lcm(a,b)
  return a*b/gcd(a,b)
end

def gcds(a)
  if (a.size==2)
    return gcd(a[0],a[1])
  else
    return gcd(a[-1], gcds(a[0..-2]))
  end
end

def lcms(a)
  if  (a.size==2)
    return lcm(a[0],a[1])
  else
    return lcm(a[-1], lcms(a[0..-2]))
  end
end

def sum2tex(a,b,pm)
  cd = a.denominator*b.denominator/gcd(a.denominator,b.denominator)
  return a.to_tex(cd/a.denominator)+%w(- +)[pm]+b.to_tex(cd/b.denominator)
end

def randRat(nmin, nmax, dmin, dmax)
  r = nil
  50.times{
    r = Rational(rand(nmax+1)+nmin, rand(dmax+1)+dmin)
    if r.denominator>=dmin && r.numerator>=nmin
      r*=(rand(2)*2-1)
      return r
    end
  }
  return r
end

def hasSquare?(i)
   r=2;
   while (r*r<=i)
       if i % (r*r) == 0
           return true
       end
       r+=1
   end
   return false
end

def wurzel(i)
   c = 1
   r=2
   while (r*r<=i)
       if (i% (r*r) == 0)
          c*=r
          i/=(r*r)
       else
           r+=1
       end
   end
   return c,i       
end

def exVectWurzel(n=10)
    while true
        v = Array.new(3) {randRat(2,8,2,9)}
        q = v.map{|x| x*x}
        l2 = q.sum
        a,b = l2.numerator, l2.denominator
        if (a<=100 && b<=100)
            if (hasSquare?(a) && hasSquare?(b))
                c1,w1 = wurzel(a)
                c2,w2 = wurzel(b)
                if (w1>1 && w2>1)
                    c = Rational(c1,c2*w2)
                    w = w1*w2
                    if (w<100 && c.denominator.abs<=100 && c.numerator.abs<=100)
                        ex = "$\\begin{pmatrix}" + v.map{|x| x.to_tex}.join("\\\\ ")+" \\end{pmatrix}$"
                        sol = "$\\sqrt{"+q.map{|x| x.to_tex}.join("+")+"} = "
                        sol += "\\sqrt{#{l2.to_tex}} = #{Rational(c1,c2).to_tex} \\sqrt{#{Rational(w1,w2).to_tex}} = "
                        sol += "#{Rational(c1,c2).to_tex} \\sqrt{#{Rational(w1,w2).to_tex}} \\cdot \\sqrt{\\frac{#{w2}}{#{w2}}} = #{c.to_tex} \\sqrt{#{w}}$"
        
                        return ex,sol
                    end
                end
            end
        end
    end
end

ex = ARGV[0].to_i

n = 10

if (ex==1)

  
  e = Array.new(n){exVectWurzel()}
  
  # puts text
  name = "veclenghtroots"
  res = "Berechnen Sie die Längen folgender Vektoren. Alle Wurzeln sind quadrat- und bruchfrei zu schreiben. Brüche sind vollständig zu kürzen.\n"
  res += header(name)
  res += e.inspect
  res += footer(name, " &nbsp; &nbsp; ")+"\n"

  xclip(res)
end
