# coding: utf-8
require "./header-footer.rb"

class Rational
  def to_tex(op=false)
    s = op && self>0 ? "+" : (self<0 ? "-" : "")
    if denominator==1
      return s+numerator.abs.to_s
    else
      return "#{s}\\frac{#{numerator.abs}}{#{denominator}}"
    end
  end  
end

def gcd(a,b)
  return a if b==0
  return gcd(b, a%b)
end

def lcm(a,b)
  return a*b/gcd(a,b)
end

def gcds(a)
  if (a.size==2)
    return gcd(a[0],a[1])
  else
    return gcd(a[-1], gcds(a[0..-2]))
  end
end

def lcms(a)
  if  (a.size==2)
    return lcm(a[0],a[1])
  else
    return lcm(a[-1], lcms(a[0..-2]))
  end
end


def randrat(max=9, neg=false)
  r=nil
  begin
    r = Rational(rand(max)+1, rand(max-1)+2)
  end until r.denominator!=1
  return r*(neg ? (rand(2)*2-1) : 1)
end

def randZ(min=2, max=9)
  return (rand(max-min+1)+min)*(rand(2)*2-1)
end

# Returns [alpha, a, b] for alpha*x^a * (1/x^b)
# with alpha: Fraction;  a,b in Z
def coeffs
  r = nil
  while true
    r = [randrat(), randZ(2,9), randZ(2,9)]
    break if (r[1]-r[2]).abs>3
  end    
  if r[1]-r[2]<0
    r=[r[0],r[2],r[1]]
  end
  return r
end

def c2e(c, op=false)
  return "#{c[0].to_tex(op)} \\cdot x^{#{c[1]}} \\cdot \\frac{1}{x^{#{c[2]}}}"
end

def c2e2(c,op=false)
  return "#{c[0].to_tex(op)}" + (c[1]!=0 ?  "\\cdot x^{#{c[1]}}" : "")
end

def makeex
  while true
    e = [coeffs(), coeffs(), coeffs(), coeffs()]
    r = e.map{|c| [c[0], c[1]-c[2]]}
    t1 = r[1][1]-r[0][1]
    t2 = r[3][1]-r[2][1]
    d = t2-t1
    e[3][2]+=d
    e[3][0] = e[2][0]/e[0][0]*e[1][0]
    if e[0][0]!=e[2][0] &&  r[0]!=r[1] && r[0]!=0 && r[1]!=0

      ex = "\\frac{#{c2e(e[0])} #{c2e(e[1],true)}}{#{c2e(e[2])} #{c2e(e[3],true)}}"
      e.map!{|c| [c[0], c[1]-c[2]]}

      if ((e[0][1]-e[1][1]).abs>1)
      
        sol = "\\frac{#{c2e2(e[0])} #{c2e2(e[1],true)}}{#{c2e2(e[2])} #{c2e2(e[3],true)}} = "
        minx = [[e[0][1], e[1][1]].min, [e[2][1], e[3][1]].min]
        frac = Array.new(2){|i|
          Rational(gcd(e[2*i][0].numerator.abs, e[2*i+1][0].numerator.abs),lcm(e[2*i][0].denominator, e[2*i+1][0].denominator))
        }
        if frac.map{|f| f.denominator}.max<50
          4.times{|i|
            e[i][1] -= minx[i/2]
            e[i][0] /= frac[i/2]
          }
          
          sol += "\\frac{#{frac[0].to_tex()}\\cdot x^{#{minx[0]}} \\cdot \\left(#{c2e2(e[0])} #{c2e2(e[1],true)}\\right)}{#{frac[1].to_tex()}\\cdot x^{#{minx[1]}} \\cdot \\left(#{c2e2(e[2])} #{c2e2(e[3],true)}\\right)} = "
          c = frac[0]/frac[1]
          e = minx[0]-minx[1]
          if (e.abs>1)
            sol += "#{frac[0].to_tex()} \\cdot #{(1/frac[1]).to_tex()} \\cdot x^{#{e}} = "
            sol += "#{c.to_tex()}\\cdot x^{#{e}}"
            
            return ["$\\displaystyle #{ex}$", "$\\displaystyle #{sol}$"]
          end
        end
      end
    end
  end
end

def pofer(coeff, exps, op=false)
    res = ""
    if coeff==1
        if exps.max==0
            return "1"
        end
    elsif coeff==-1
        if exps.max==0
            return "-1"
        end
        res = "-"
    else
        res = "#{coeff.to_tex(op)}"
    end
    bases = ["x", "y"]
    exps.each.with_index{|e,i|
        if e>0  
            res += "#{bases[i]}"
            if e>0
                res+="^{#{e}}"
            end
        end
    }
    return res
end

# Form ax^by^c + dx^ey^f
def makeexausklammern()
    while true
        k = [randrat(9,true), randZ(), randZ(), randrat(9,true), randZ(4,20), randZ()]
        if k[0].denominator!=k[3].denominator && k[0].numerator!= k[3].numerator && gcd(k[0].numerator.abs,k[3].numerator.abs) > 1 && [1]!=k[4] && k[2]!=k[5] 
            ex = "$\\displaystyle #{k[0].to_tex()} \\cdot x^{#{k[1]}} y^{#{k[2]}} #{k[3].to_tex(true)} \\cdot x^{#{k[4]}}y^{#{k[5]}}$"
            nenner = lcm(k[0].denominator,k[3].denominator)
            zaehler = gcd(k[0].numerator.abs,k[3].numerator.abs)
            k[0]*=nenner
            k[3]*=nenner
            k[0]/=zaehler
            k[3]/=zaehler
            sol = "$\\displaystyle \\frac{#{zaehler}}{#{nenner}}\\left(#{k[0].to_tex()} \\cdot x^{#{k[1]}} y^{#{k[2]}} #{k[3].to_tex(true)} \\cdot x^{#{k[4]}}y^{#{k[5]}}\\right)="
            xmin = [k[1],k[4]].min
            ymin = [k[2],k[5]].min
            m = Rational(zaehler, nenner)
            if m.denominator>1
                k[1]-=xmin
                k[2]-=ymin
                k[4]-=xmin
                k[5]-=ymin
                
                sol += "#{m.to_tex()}x^{#{xmin}}y^{#{ymin}}\\left(#{pofer(k[0],[k[1],k[2]])}#{pofer(k[3],[k[4],k[5]], true)}\\right)$"
                return [ex,sol]
            end
        end
    end
end

ex,name,res = nil,nil,nil

if ARGV[0].to_i==2
    ex = Array.new(10){makeexausklammern()}
    name = "ausklammern"
    res = "Klammern Sie die jeweils kleinste Potenz von $x$ und $y$ aus, sowie ein Bruch so, dass in der Klammer kleinstmögliche ganze Zahlen als Koeffizienten vorkommen.\n"
else

    ex = Array.new(10){makeex()}
    name = "vereinfachenAusklammern"
    res = "Zusammenfassen, ausklammern, kürzen, Resultat als Vielfaches einer Potenz von $x$:\n"
end

res += header(name)
res += ex.inspect
res += footer(name, " <br><hr> ", " <br><hr> ", 3)
xclip(res)
