# coding: utf-8
require "matrix"
require "./header-footer.rb"


class Rational
  def to_tex(amplify=1, paren=false)
    p=(paren && (self<0)) ? ["\\left(", "\\right)"] : ["",""]
    if denominator==1
      return p[0]+numerator.to_s+p[1]
    else
      s = numerator<0 ? "-" : ""
      return "#{p[0]}#{s}\\frac{#{numerator.abs*amplify}}{#{denominator*amplify}}#{p[1]}"
    end
  end  
end


class Vector
  def to_tex(vec=true)
    return "\\begin{pmatrix} "+self.map{|x| x}.to_a.join(" \\\\ ")+"\\end{pmatrix} " if vec
    return "\\left("+self.map{|x| x}.to_a.join(",\\,")+"\\right)"
  end
end

def tripel(max)
    res = []
    for x in 1..max
        for y in x..max
            for z in y..max
                l2 = x*x+y*y+z*z
                if l2 == (Math.sqrt(l2).to_i)**2
                    res.push(Vector.elements([x,y,z]))
                end
            end
        end
    end
    return res
end



def makeex(trip)
    while true
        v = Array.new(2){
            Vector.elements(trip[rand(trip.size)].to_a.shuffle.map{|e| e*(rand(2)*2-1)})
        }
        dp = v[0].inner_product(v[1])
        next if  dp == 0
        l = v.map{|x| x.magnitude().to_i}
        cs = Rational(dp, l[0]*l[1])
        next if cs.denominator>10
        next if cs==-1 || cs==1
        csa = (cs.to_f*100+0.5).to_i*0.01
        w = (Math.acos(cs)/Math::PI*180+0.5).to_i
        ex = "$\\vec u = #{v[0].to_tex}, \\qquad \\vec v = #{v[1].to_tex}$"
        sol = "$\\cos(\\alpha) = \\frac{\\vec u \\cdot \\vec v}{|\\vec v| \\cdot |\\vec u|} = \\frac{#{dp}}{#{l[0]} \\cdot #{l[1]}} = #{cs.to_tex} \\approx #{csa}$. "
        sol += "Auf dem Einheitskreis liest man ungefähr den zugehörigen Winkel ab: #{w}$^\\circ$"
        puts "#{cs} -> #{w}"
        return [ex,sol]
    end
end
trip = tripel(10);

e = Array.new(10){makeex(trip)}

name = "winkelzwischenvektorenvonhand"
res = "Bestimmen Sie von Hand den ungefähren Winkel (auf $15^\\circ$ genau) zwischen den Vektoren $\\vec u$ und $\\vec v$. Machen Sie eine Skizze im Einheitskreis, um den Winkel zum errechneten Cosinus-Wert zu bestimmen."
res += header(name)
res += e.inspect
res += footer(name, "<br><hr>", "<br><hr>")+"\n"

xclip(res)

